@@ -367,9 +367,12 @@ activate_port(Jalv*    jalv,
 			JACK_DEFAULT_AUDIO_TYPE, jack_flags, 0);
 		break;
 	case TYPE_EVENT:
-		port->jack_port = jack_port_register(
-			jalv->jack_client, symbol_str,
-			JACK_DEFAULT_MIDI_TYPE, jack_flags, 0);
+		if (lilv_port_supports_event(
+			    jalv->plugin, port->lilv_port, jalv->nodes.midi_MidiEvent)) {
+			port->jack_port = jack_port_register(
+				jalv->jack_client, symbol_str,
+				JACK_DEFAULT_MIDI_TYPE, jack_flags, 0);
+		}
 		break;
 	default:
 		break;
@@ -472,10 +475,7 @@ jack_process_cb(jack_nframes_t nframes, void* data)
 	/* Prepare port buffers */
 	for (uint32_t p = 0; p < jalv->num_ports; ++p) {
 		struct Port* port = &jalv->ports[p];
-		if (!port->jack_port)
-			continue;
-
-		if (port->type == TYPE_AUDIO) {
+		if (port->type == TYPE_AUDIO && port->jack_port) {
 			/* Connect plugin port directly to Jack port buffer */
 			lilv_instance_connect_port(
 				jalv->instance, p,
@@ -492,15 +492,17 @@ jack_process_cb(jack_nframes_t nframes, void* data)
 					lv2_pos->type, lv2_pos->size, LV2_ATOM_BODY(lv2_pos));
 			}
 
-			/* Write Jack MIDI input */
-			void* buf = jack_port_get_buffer(port->jack_port, nframes);
-			for (uint32_t i = 0; i < jack_midi_get_event_count(buf); ++i) {
-				jack_midi_event_t ev;
-				jack_midi_event_get(&ev, buf, i);
-				lv2_evbuf_write(&iter,
-				                ev.time, 0,
-				                jalv->midi_event_id,
-				                ev.size, ev.buffer);
+			if (port->jack_port) {
+				/* Write Jack MIDI input */
+				void* buf = jack_port_get_buffer(port->jack_port, nframes);
+				for (uint32_t i = 0; i < jack_midi_get_event_count(buf); ++i) {
+					jack_midi_event_t ev;
+					jack_midi_event_get(&ev, buf, i);
+					lv2_evbuf_write(&iter,
+					                ev.time, 0,
+					                jalv->midi_event_id,
+					                ev.size, ev.buffer);
+				}
 			}
 		} else if (port->type == TYPE_EVENT) {
 			/* Clear event output for plugin to write to */
@@ -527,8 +529,8 @@ jack_process_cb(jack_nframes_t nframes, void* data)
 			} else if (ev.protocol == jalv->urids.atom_eventTransfer) {
 				LV2_Evbuf_Iterator    e    = lv2_evbuf_end(port->evbuf);
 				const LV2_Atom* const atom = (const LV2_Atom*)body;
-				lv2_evbuf_write(&e, nframes, 0,
-				                atom->type, atom->size, LV2_ATOM_BODY(atom));
+				lv2_evbuf_write(&e, nframes, 0, atom->type, atom->size,
+				                LV2_ATOM_BODY_CONST(atom));
 			} else {
 				fprintf(stderr, "error: Unknown control change protocol %d\n",
 				        ev.protocol);
@@ -559,10 +561,12 @@ jack_process_cb(jack_nframes_t nframes, void* data)
 	/* Deliver MIDI output and UI events */
 	for (uint32_t p = 0; p < jalv->num_ports; ++p) {
 		struct Port* const port = &jalv->ports[p];
-		if (port->jack_port && port->flow == FLOW_OUTPUT
-		    && port->type == TYPE_EVENT) {
-			void* buf = jack_port_get_buffer(port->jack_port, nframes);
-			jack_midi_clear_buffer(buf);
+		if (port->flow == FLOW_OUTPUT && port->type == TYPE_EVENT) {
+			void* buf = NULL;
+			if (port->jack_port) {
+				buf = jack_port_get_buffer(port->jack_port, nframes);
+				jack_midi_clear_buffer(buf);
+			}
 
 			for (LV2_Evbuf_Iterator i = lv2_evbuf_begin(port->evbuf);
 			     lv2_evbuf_is_valid(i);
@@ -570,7 +574,7 @@ jack_process_cb(jack_nframes_t nframes, void* data)
 				uint32_t frames, subframes, type, size;
 				uint8_t* body;
 				lv2_evbuf_get(i, &frames, &subframes, &type, &size, &body);
-				if (type == jalv->midi_event_id) {
+				if (buf && type == jalv->midi_event_id) {
 					jack_midi_event_write(buf, frames, body, size);
 				}
 
@@ -644,7 +648,7 @@ jack_session_cb(jack_session_event_t* event, void* arg)
 void
 jalv_ui_instantiate(Jalv* jalv, const char* native_ui_type, void* parent)
 {
-	jalv->ui_host = suil_host_new(jalv_ui_write, NULL, NULL, NULL);
+	jalv->ui_host = suil_host_new(jalv_ui_write, jalv_ui_port_index, NULL, NULL);
 
 	const LV2_Feature parent_feature = {
 		LV2_UI__parent, parent
@@ -733,7 +737,7 @@ jalv_ui_write(SuilController controller,
 		const LV2_Atom* atom = (const LV2_Atom*)buffer;
 		char*           str  = sratom_to_turtle(
 			jalv->sratom, &jalv->unmap, "jalv:", NULL, NULL,
-			atom->type, atom->size, LV2_ATOM_BODY(atom));
+			atom->type, atom->size, LV2_ATOM_BODY_CONST(atom));
 		printf("\n## UI => Plugin (%u bytes) ##\n%s\n", atom->size, str);
 		free(str);
 	}
@@ -747,6 +751,15 @@ jalv_ui_write(SuilController controller,
 	jack_ringbuffer_write(jalv->ui_events, buf, sizeof(buf));
 }
 
+uint32_t
+jalv_ui_port_index(SuilController controller, const char* symbol)
+{
+	Jalv* const  jalv = (Jalv*)controller;
+	struct Port* port = jalv_port_by_symbol(jalv, symbol);
+
+	return port ? port->index : LV2UI_INVALID_PORT_INDEX;
+}
+
 bool
 jalv_emit_ui_events(Jalv* jalv)
 {
@@ -786,6 +799,26 @@ jalv_emit_ui_events(Jalv* jalv)
 	return true;
 }
 
+static bool
+jalv_apply_control_arg(Jalv* jalv, const char* s)
+{
+	char  sym[256];
+	float val = 0.0f;
+	if (sscanf(s, "%[^=]=%f", sym, &val) != 2) {
+		fprintf(stderr, "warning: Ignoring invalid value `%s'\n", s);
+		return false;
+	}
+	
+	struct Port* port = jalv_port_by_symbol(jalv, sym);
+	if (!port) {
+		fprintf(stderr, "warning: Ignoring value for unknown port `%s'\n", sym);
+		return false;
+	}
+
+	port->control = val;
+	return true;
+}
+
 static void
 signal_handler(int ignored)
 {
@@ -1102,7 +1135,7 @@ main(int argc, char** argv)
 	    && lilv_plugin_has_extension_data(jalv.plugin, jalv.nodes.work_interface)) {
 		jalv_worker_init(
 			&jalv, &jalv.worker,
-			(LV2_Worker_Interface*)lilv_instance_get_extension_data(
+			(const LV2_Worker_Interface*)lilv_instance_get_extension_data(
 				jalv.instance, LV2_WORKER__interface));
 	}
 
@@ -1111,6 +1144,12 @@ main(int argc, char** argv)
 		jalv_apply_state(&jalv, state);
 	}
 
+	if (jalv.opts.controls) {
+		for (char** c = jalv.opts.controls; *c; ++c) {
+			jalv_apply_control_arg(&jalv, *c);
+		}
+	}
+
 	/* Set Jack callbacks */
 	jack_set_process_callback(jalv.jack_client,
 	                          &jack_process_cb, (void*)(&jalv));
