@@ -87,12 +87,18 @@ jalv_init(int* argc, char*** argv, JalvOptions* opts)
 		  "Load state from save directory", "DIR" },
 		{ "dump", 'd', 0, G_OPTION_ARG_NONE, &opts->dump,
 		  "Dump plugin <=> UI communication", NULL },
+		{ "show-hidden", 's', 0, G_OPTION_ARG_NONE, &opts->show_hidden,
+		  "Show controls for ports with notOnGUI property on generic UI", NULL },
+		{ "no-menu", 'n', 0, G_OPTION_ARG_NONE, &opts->no_menu,
+		  "Do not show Jalv menu on window", NULL },
 		{ "generic-ui", 'g', 0, G_OPTION_ARG_NONE, &opts->generic_ui,
 		  "Use Jalv generic UI and not the plugin UI", NULL},
 		{ "buffer-size", 'b', 0, G_OPTION_ARG_INT, &opts->buffer_size,
 		  "Buffer size for plugin <=> UI communication", "SIZE"},
 		{ "update-frequency", 'r', 0, G_OPTION_ARG_DOUBLE, &opts->update_rate,
 		  "UI update frequency", NULL},
+		{ "control", 'c', 0, G_OPTION_ARG_STRING_ARRAY, &opts->controls,
+		  "UI update frequency", NULL},
 		{ 0, 0, 0, 0, 0, 0, 0 } };
 	GError* error = NULL;
 	const int err = gtk_init_with_args(
@@ -274,7 +280,7 @@ jalv_ui_port_event(Jalv*       jalv,
 
 	if (controller->spin) {
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(controller->spin),
-		                          *(float*)buffer);
+		                          *(const float*)buffer);
 	}
 
 	GtkWidget* widget = controller->control;
@@ -287,7 +293,7 @@ jalv_ui_port_event(Jalv*       jalv,
 			gtk_tree_model_get_value(model, &i, 0, &value);
 			const double v = g_value_get_double(&value);
 			g_value_unset(&value);
-			if (fabs(v - *(float*)buffer) < FLT_EPSILON) {
+			if (fabs(v - *(const float*)buffer) < FLT_EPSILON) {
 				gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widget), &i);
 				return;
 			}
@@ -295,9 +301,9 @@ jalv_ui_port_event(Jalv*       jalv,
 		}
 	} else if (GTK_IS_TOGGLE_BUTTON(widget)) {
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
-		                             *(float*)buffer > 0.0f);
+		                             *(const float*)buffer > 0.0f);
 	} else if (GTK_IS_RANGE(widget)) {
-		gtk_range_set_value(GTK_RANGE(widget), *(float*)buffer); 
+		gtk_range_set_value(GTK_RANGE(widget), *(const float*)buffer); 
 	} else {
 		fprintf(stderr, "Unknown widget type for port %d\n", port_index);
 	}
@@ -400,16 +406,16 @@ file_changed(GtkFileChooserButton* widget,
 static gint
 dcmp(gconstpointer a, gconstpointer b)
 {
-	double y = *(double*)a;
-	double z = *(double*)b;
+	double y = *(const double*)a;
+	double z = *(const double*)b;
 	return y < z ? -1 : z < y ? 1 : 0;
 }
 
 static gint
 drcmp(gconstpointer a, gconstpointer b)
 {
-	double y = *(double*)a;
-	double z = *(double*)b;
+	double y = *(const double*)a;
+	double z = *(const double*)b;
 	return y < z ? 1 : z < y ? -1 : 0;
 }
 
@@ -608,6 +614,7 @@ build_control_widget(Jalv* jalv, GtkWidget* window)
 	LilvNode*   lv2_toggled     = lilv_new_uri(world, LV2_CORE__toggled);
 	LilvNode*   patch_writable  = lilv_new_uri(world, LV2_PATCH__writable);
 	LilvNode*   rdfs_comment    = lilv_new_uri(world, LILV_NS_RDFS "comment");
+	LilvNode*   pprop_notOnGUI  = lilv_new_uri(world, LV2_PORT_PROPS__notOnGUI);
 	GtkWidget*  port_table      = gtk_table_new(jalv->num_ports, 3, false);
 
 	/* Get the min and max of all ports (or NaN if unavailable) */
@@ -629,6 +636,12 @@ build_control_widget(Jalv* jalv, GtkWidget* window)
 	int       n_rows     = 0;
 	for (unsigned i = 0; i < control_ports->len; ++i) {
 		const LilvPort* port  = g_array_index(control_ports, LilvPort*, i);
+
+		if (!jalv->opts.show_hidden &&
+		    lilv_port_has_property(plugin, port, pprop_notOnGUI)) {
+			continue;
+		}
+
 		uint32_t        index = lilv_port_get_index(plugin, port);
 		LilvNode*       name  = lilv_port_get_name(plugin, port);
 
@@ -729,6 +742,7 @@ build_control_widget(Jalv* jalv, GtkWidget* window)
 	lilv_node_free(lv2_sampleRate);
 	lilv_node_free(lv2_integer);
 	lilv_node_free(lv2_enumeration);
+	lilv_node_free(pprop_notOnGUI);
 	lilv_node_free(logarithmic);
 
 	if (n_rows > 0) {
@@ -747,21 +761,9 @@ build_control_widget(Jalv* jalv, GtkWidget* window)
 	}
 }
 
-int
-jalv_open_ui(Jalv* jalv)
+static void
+build_menu(Jalv* jalv, GtkWidget* window, GtkWidget* vbox)
 {
-	GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	jalv->window = window;
-	jalv->has_ui = TRUE;
-
-	g_signal_connect(window, "destroy",
-	                 G_CALLBACK(on_window_destroy), jalv);
-
-	LilvNode* name = lilv_plugin_get_name(jalv->plugin);
-	gtk_window_set_title(GTK_WINDOW(window), lilv_node_as_string(name));
-	lilv_node_free(name);
-
-	GtkWidget* vbox      = new_box(false, 0);
 	GtkWidget* menu_bar  = gtk_menu_bar_new();
 	GtkWidget* file      = gtk_menu_item_new_with_mnemonic("_File");
 	GtkWidget* file_menu = gtk_menu_new();
@@ -789,10 +791,6 @@ jalv_open_ui(Jalv* jalv)
 
 	jalv_load_presets(jalv, add_preset_to_menu, presets_menu);
 
-	gtk_window_set_role(GTK_WINDOW(window), "plugin_ui");
-	gtk_container_add(GTK_CONTAINER(window), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);
-
 	g_signal_connect(G_OBJECT(quit), "activate",
 	                 G_CALLBACK(on_quit_activate), window);
 
@@ -802,6 +800,31 @@ jalv_open_ui(Jalv* jalv)
 	g_signal_connect(G_OBJECT(save_preset), "activate",
 	                 G_CALLBACK(on_save_preset_activate), jalv);
 
+	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);
+}
+
+int
+jalv_open_ui(Jalv* jalv)
+{
+	GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	jalv->window = window;
+	jalv->has_ui = TRUE;
+
+	g_signal_connect(window, "destroy",
+	                 G_CALLBACK(on_window_destroy), jalv);
+
+	LilvNode* name = lilv_plugin_get_name(jalv->plugin);
+	gtk_window_set_title(GTK_WINDOW(window), lilv_node_as_string(name));
+	lilv_node_free(name);
+
+	GtkWidget* vbox = new_box(false, 0);
+	gtk_window_set_role(GTK_WINDOW(window), "plugin_ui");
+	gtk_container_add(GTK_CONTAINER(window), vbox);
+
+	if (!jalv->opts.no_menu) {
+		build_menu(jalv, window, vbox);
+	}
+
 	/* Create/show alignment to contain UI (whether custom or generic) */
 	GtkWidget* alignment = gtk_alignment_new(0.5, 0.5, 1.0, 1.0);
 	gtk_box_pack_start(GTK_BOX(vbox), alignment, TRUE, TRUE, 0);
